pro misr_time

;  This program calculates block center times for a MISR Level 1B2 file
;  using the RANGE_BEGINNING_DATE and RANGE_ENDING_DATE from the
;  file's core metadata.  It returns values to the nearest second, and
;  in comparison with block center times stored in files prodouced
;  beginning in August 2003, these values are with one second of the
;  results calculated during processing.  The block center data file
;  is created with the same name as the MISR L1B2 file, with the addition
;  to the file name of _block_centers.txt and without the original file's
;  .hdf suffix.  The block center file is written to the current IDL directory.
;
;  To use this program, untar or unzip the misr_time.tar file into a
;  directory.  Start the IDL development environment and from the idlde
;  file menu, open the project misr_time.prj which is located in the
;  misr_time directory.  From the idlde Project menu, choose Build and
;  then choose Run.
;
;  A .sav file is also included in the misr_time tar package which can be
;  run using the IDL Virtual Machine.  The IDL Virtual Machine is a free
;  version of IDL available from Research Systems, Inc. at
;  http://www/rsinc.com/idlvm/.  The IDL VM software may be downloaded from
;  this site or ordered from RSI on CD at no cost.  The site provides
;  installation instructions.
;
;  NOTES:
;  1)  A time of 00-00-0000 00:00:00 UTC for the start block center indicates
;  that data in the block start after the block center.  Likewise, if the
;  end block center is 00-00-0000 00:00:00 UTC, data in the end block end
;  before the block center.
;
;  The following notes apply to those working with a fully licensed
;  version of IDL:
;  2)  This program is written to work with Level 1B2 data in the
;  stacked block format.  To use this program with Level 2 data, it will
;  be necessary to choose an appropriate parameter to search for the
;  start of data and use the grid name and field name for that parameter
;  in variables set for the call to read_grid_field below.
;
;  3)  The program is written to work with a parameter whose fill value is
;  positive, so if a parameter with a negative fill value is used, the
;  comparisons with fill value in calculating the first good data line
;  and the last good data line would also need to be modified.
;
;  4)  The path in the DIALOG_PICKFILE call below can be edited to specify
;  the directory path for MISR data files.


mfile = DIALOG_PICKFILE(PATH="")  ; specify the directory path location of
                                  ; the MISR files in the PATH variable here

;  create output file name

fname = FILE_BASENAME(mfile,'.hdf')
bcfile = fname + '_block_centers.txt'

status = get_core_range_date_time(mfile,stdate,sttime,enddate,endtime)
get_block_info,mfile,stblock,endblock,corners
fillvalue = 1

;  change grid name and field name here if using a MISR file of type other
;  than Level 1B2

grid = "RedBand"
field = "Red Radiance/RDQI"
status = read_grid_field(mfile,grid,field,data,FILLVALUE=fillvalue)
dims = SIZE(data,/DIMENSIONS)

; find the first line with good data in the start block

good = where(data[*,*,stblock-1] lt fillvalue,gcount) ; modify if fill value is lt 0
if (gcount lt 0) then begin
  response = DIALOG_MESSAGE("***No good points found in start block")
  goto, exit
endif else begin
  ;print,"First good point: ",good[0]
  first_good_line = fix(good[0]/dims[0])+1
  ;print,"First good line: ",first_good_line
endelse

; find the last line with good data in the end block

good = where(data[*,*,endblock-1] lt fillvalue,gcount) ; modify if fill value is lt 0
if gcount lt 0 then begin
  response = DIALOG_MESSAGE("***No good points found in end block")
  goto, exit
endif else begin
  ;print,"Last good point: ",good[n_elements(good)-1]
  last_good_line = fix(good[n_elements(good)-1]/dims[0])+1
  ;print,"Last good line: ",last_good_line
endelse

; calculate the block center times

whole_blocks = endblock - stblock - 1
total_lines = whole_blocks*dims[1] + dims[1]-first_good_line+1 + last_good_line
stjulday = misr_calc_jul_date(stdate,sttime)
endjulday = misr_calc_jul_date(enddate,endtime)
deltatime = endjulday - stjulday
time_per_line = deltatime/total_lines

block_center_timestamp = strarr(endblock)
block_center_jultime = dblarr(endblock)

for i=0,stblock-2 do begin
  block_center_timestamp[i] = ""
  block_center_jultime[i] = 0
endfor
if (first_good_line ge dims[1]/2) then begin
  block_center_timestamp[stblock-1] = '00-00-0000 00:00:00 UTC'
  block_center_jultime[stblock-1] = stjulday - (first_good_line - dims[1]/2)*time_per_line
endif else begin
  block_center_jultime[stblock-1] = (dims[1]/2-first_good_line)*time_per_line + stjulday
  block_center_timestamp[stblock-1] = misr_julday_to_timestamp(block_center_jultime[stblock-1])
  endelse
for i=stblock,endblock-2 do begin
  block_center_jultime[i] = block_center_jultime[i-1] + time_per_line*dims[1]
  block_center_timestamp[i] = misr_julday_to_timestamp(block_center_jultime[i])
endfor
if (last_good_line le (dims[1]/2)-1) then begin
  block_center_timestamp[endblock-1] = '00-00-0000 00:00:00 UTC'
endif else begin
  block_center_jultime[endblock-1] = block_center_jultime[endblock-2] + time_per_line*dims[1]
  block_center_timestamp[endblock-1] = misr_julday_to_timestamp(block_center_jultime[endblock-1])
endelse

OPENW,outlun,bcfile,/GET_LUN
printf,outlun,"       Block   Center Time"
for i=stblock-1,endblock-1 do printf,outlun,i+1,"   ",block_center_timestamp[i]
FREE_LUN,outlun

result = DIALOG_MESSAGE("Block time center calculation is complete",/INFORMATION)

exit:

end
